@@ -1,7 +1,7 @@
 /*
  * ======= General internationalization, for Lisp programs too =======
  * Copyright (C) 1990-2005 Bruno Haible
- * Copyright (C) 1998-2005 Sam Steingold
+ * Copyright (C) 1998-2006 Sam Steingold
  * GPL2
  */
 
@@ -18,6 +18,7 @@
 #if defined(HAVE_LANGINFO_H)
 # include <langinfo.h>
 #endif
+#include <limits.h>            /* for CHAR_MAX */
 
 #ifdef CLISP_UNICODE
 # define if_UNICODE(statement)  statement
@@ -181,13 +182,13 @@ DEFUN(I18N:TEXTDOMAINDIR, domain)
 
 DEFUN(I18N:SET-TEXTDOMAINDIR, domain directory)
 { /* sets the message catalog directory for the given domain. */
-  object domain = check_string(STACK_1);
+  object domain = (STACK_1=check_string(STACK_1));
  #ifdef GNU_GETTEXT
   /* Check and use default directory, because the bindtextdomain()
      documentation recommends that the argument be an absolute pathname,
      to protect against later chdir() calls. */
   object directory = pathname_to_OSdir(STACK_0,true);
-  with_string_0(domain,Symbol_value(S(ascii)),domain_asciz, {
+  with_string_0(STACK_1/*domain*/,Symbol_value(S(ascii)),domain_asciz, {
     begin_system_call();
     bindtextdomain(domain_asciz,TheAsciz(directory));
     end_system_call();
@@ -208,26 +209,26 @@ DEFUN(I18N:SET-LOCALE, &optional category locale)
   if (missingp(*category)) {
     int pos = 0;
     if (missingp(*locale)) {
-      for (; pos < check_locale_category_table_size; pos++) {
+      for (; pos < check_locale_category_map.size; pos++) {
         begin_system_call();
-        res = setlocale(check_locale_category_table[pos].c_const,NULL);
+        res = setlocale(check_locale_category_map.table[pos].c_const,NULL);
         end_system_call();
-        pushSTACK(*check_locale_category_table[pos].l_const);
+        pushSTACK(*check_locale_category_map.table[pos].l_const);
         pushSTACK(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
       }
     } else {
       *locale = check_string(*locale);
       with_string_0(*locale,GLO(misc_encoding),loc_z,{
-          for (; pos < check_locale_category_table_size; pos++) {
+          for (; pos < check_locale_category_map.size; pos++) {
             begin_system_call();
-            res = setlocale(check_locale_category_table[pos].c_const,loc_z);
+            res = setlocale(check_locale_category_map.table[pos].c_const,loc_z);
             end_system_call();
-            pushSTACK(*check_locale_category_table[pos].l_const);
+            pushSTACK(*check_locale_category_map.table[pos].l_const);
             pushSTACK(res ? asciz_to_string(res,GLO(misc_encoding)) : NIL);
           }
         });
     }
-    VALUES1(listof(2*check_locale_category_table_size));
+    VALUES1(listof(2*check_locale_category_map.size));
   } else {
     int cat_value = check_locale_category(*category);
     if (missingp(*locale)) {
@@ -275,15 +276,18 @@ static void thousands_sep_to_STACK (char* sep1000) {
   for (ii=0; sep1000[ii]; ii++) pushSTACK(fixnum(sep1000[ii]));
   pushSTACK(Fixnum_0); value1 = vectorof(ii+1); pushSTACK(value1);
 }
-#if !defined(CHAR_MAX)
-# define CHAR_MAX 127
-#endif
-static object bool_char_lconv(char val) {
+static /*maygc*/ object bool_char_lconv(char val) {
   switch (val) {
     case 0: return NIL;
     case 1: return T;
     case CHAR_MAX: return S(Kunspecific);
-    default: NOTREACHED;
+    default:
+      pushSTACK(CLSTEXT("~S: localeconv() returned an invalid value ~S (should be one of ~S, ~S, CHAR_MAX=~S)"));
+      pushSTACK(TheSubr(subr_self)->name);
+      pushSTACK(fixnum(val));
+      pushSTACK(Fixnum_0); pushSTACK(Fixnum_1); pushSTACK(fixnum(CHAR_MAX));
+      funcall(S(warn),6);
+      return fixnum(val);
   }
 }
 static object int_char_lconv(char val) {
@@ -309,11 +313,11 @@ DEFUN(I18N:LOCALE-CONV,)
   pushSTACK(int_char_lconv(lc->int_frac_digits));
   pushSTACK(int_char_lconv(lc->frac_digits));
   pushSTACK(bool_char_lconv(lc->p_cs_precedes));
-  pushSTACK(bool_char_lconv(lc->p_sep_by_space));
+  pushSTACK(int_char_lconv(lc->p_sep_by_space));
   pushSTACK(bool_char_lconv(lc->n_cs_precedes));
-  pushSTACK(bool_char_lconv(lc->n_sep_by_space));
-  pushSTACK(bool_char_lconv(lc->p_sign_posn));
-  pushSTACK(bool_char_lconv(lc->n_sign_posn));
+  pushSTACK(int_char_lconv(lc->n_sep_by_space));
+  pushSTACK(int_char_lconv(lc->p_sign_posn));
+  pushSTACK(int_char_lconv(lc->n_sign_posn));
 #if HAVE_STRUCT_LCONV_INT_P_CS_PRECEDES
   pushSTACK(bool_char_lconv(lc->int_p_cs_precedes));
 #else
@@ -325,22 +329,22 @@ DEFUN(I18N:LOCALE-CONV,)
   pushSTACK(S(Kunspecific));
 #endif
 #if HAVE_STRUCT_LCONV_INT_P_SEP_BY_SPACE
-  pushSTACK(bool_char_lconv(lc->int_p_sep_by_space));
+  pushSTACK(int_char_lconv(lc->int_p_sep_by_space));
 #else
   pushSTACK(S(Kunspecific));
 #endif
 #if HAVE_STRUCT_LCONV_INT_N_SEP_BY_SPACE
-  pushSTACK(bool_char_lconv(lc->int_n_sep_by_space));
+  pushSTACK(int_char_lconv(lc->int_n_sep_by_space));
 #else
   pushSTACK(S(Kunspecific));
 #endif
 #if HAVE_STRUCT_LCONV_INT_P_SIGN_POSN
-  pushSTACK(bool_char_lconv(lc->int_p_sign_posn));
+  pushSTACK(int_char_lconv(lc->int_p_sign_posn));
 #else
   pushSTACK(S(Kunspecific));
 #endif
 #if HAVE_STRUCT_LCONV_INT_N_SIGN_POSN
-  pushSTACK(bool_char_lconv(lc->int_n_sign_posn));
+  pushSTACK(int_char_lconv(lc->int_n_sign_posn));
 #else
   pushSTACK(S(Kunspecific));
 #endif
@@ -485,13 +489,13 @@ DEFUNR(I18N:LANGUAGE-INFORMATION,&optional item)
   if (missingp(what)) {         /* everything */
     int pos = 0;
     DECLARE_RES;
-    for (; pos < check_nl_item_table_size; pos++) {
-      get_lang_info(check_nl_item_table[pos].c_const);
-      pushSTACK(*check_nl_item_table[pos].l_const);
+    for (; pos < check_nl_item_map.size; pos++) {
+      get_lang_info(check_nl_item_map.table[pos].c_const);
+      pushSTACK(*check_nl_item_map.table[pos].l_const);
       pushSTACK(res_to_obj());
     }
     FINISH_RES;
-    VALUES1(listof(2*check_nl_item_table_size));
+    VALUES1(listof(2*check_nl_item_map.size));
   } else {
     int item = check_nl_item(what);
     DECLARE_RES;
