@@ -89,6 +89,9 @@
 #if defined(HAVE_SYS_UIO_H)
 # include <sys/uio.h>
 #endif
+#if defined(HAVE_IFADDRS_H)
+# include <ifaddrs.h>
+#endif
 typedef SOCKET rawsock_t;
 #if SIZEOF_SIZE_T == 8
 # define size_to_I  uint64_to_I
@@ -247,11 +250,22 @@ DEFUN(RAWSOCK::SOCKADDR-SLOT,&optional slot) {
 }
 
 /* can trigger GC */
-static object make_sockaddr (void) {
-  pushSTACK(allocate_bit_vector(Atype_8Bit,sizeof(struct sockaddr)));
+static object make_sockaddr1 (uintL size) {
+  pushSTACK(allocate_bit_vector(Atype_8Bit,size));
   funcall(`RAWSOCK::MAKE-SA`,1);
   return value1;
 }
+#define make_sockaddr()  make_sockaddr1(sizeof(struct sockaddr))
+
+/* can trigger GC */
+static object sockaddr_to_lisp (struct sockaddr *sa, uintL size) {
+  pushSTACK(allocate_bit_vector(Atype_8Bit,size));
+  begin_system_call();
+  memcpy(TheSbvector(STACK_0)->data,sa,size);
+  end_system_call();
+  funcall(`RAWSOCK::MAKE-SA`,1); return value1;
+}
+#define sockaddr_to_lisp1(sa)  sockaddr_to_lisp(sa,sizeof(*sa))
 
 struct pos {
   gcv_object_t *vector;
@@ -370,6 +384,7 @@ DEFUN(RAWSOCK:NTOHS, num) {
 }
 DEFUN(RAWSOCK:CONVERT-ADDRESS, family address) {
   int family = check_socket_domain(STACK_1);
+ convert_address_restart:
   if (stringp(STACK_0)) {
     with_string_0(STACK_0,Symbol_value(S(utf_8)),ip_address,
                   { value1 = string_to_addr(ip_address); });
@@ -377,22 +392,29 @@ DEFUN(RAWSOCK:CONVERT-ADDRESS, family address) {
     switch (family) {
      #if defined(AF_INET6)
       case AF_INET6: {
-        uint64 ip_address = I_to_uint64(check_uint64(STACK_0));
+        uint64 ip_address;
+        UI_to_LEbytes(check_uint64(STACK_0),8*sizeof(uint64),
+                      (uintB*)&ip_address);
         value1 = addr_to_string(family,(char*)&ip_address);
       } break;
      #endif
       case AF_INET: {
-        uint32 ip_address = I_to_uint32(check_uint32(STACK_0));
+        uint32 ip_address;
+        UI_to_LEbytes(check_uint32(STACK_0),8*sizeof(uint32),
+                      (uintB*)&ip_address);
         value1 = addr_to_string(family,(char*)&ip_address);
       } break;
       default: value1 = NIL;
     }
   } else fehler_string_integer(STACK_0);
   if (nullp(value1)) {
+    pushSTACK(NIL);             /* no PLACE */
     pushSTACK(STACK_1);         /* domain */
     pushSTACK(STACK_1);         /* address */
     pushSTACK(TheSubr(subr_self)->name);
-    fehler(error,GETTEXT("~S: invalid address ~S for family ~S"));
+    check_value(error,GETTEXT("~S: invalid address ~S for family ~S"));
+    STACK_0 = value1;
+    goto convert_address_restart;
   }
   skipSTACK(2); mv_count = 1;
 }
@@ -505,6 +527,86 @@ DEFUN(RAWSOCK:NETWORK, &optional network type)
   else VALUES1(NIL);
 }
 #endif  /* HAVE_NETDB_H */
+/* ================== net/if.h interface ================== */
+/* http://www.opengroup.org/onlinepubs/009695399/basedefs/net/if.h.html */
+#if defined(HAVE_NET_IF_H)
+DEFUN(RAWSOCK:IF-NAME-INDEX, &optional what) {
+  if (missingp(STACK_0)) {
+    int count = 0;
+#  if defined(HAVE_IF_NAMEINDEX) && defined(HAVE_IF_FREENAMEINDEX)
+    struct if_nameindex *ifni;
+    begin_system_call();
+    if ((ifni = if_nameindex()) == NULL) OS_error();
+    end_system_call();
+    for (; ifni[count].if_index; count++) {
+      pushSTACK(allocate_cons());
+      Car(STACK_0) = uint_to_I(ifni[count].if_index);
+      Cdr(STACK_0) = asciz_to_string(ifni[count].if_name,GLO(misc_encoding));
+    }
+    begin_system_call(); if_freenameindex(ifni); end_system_call();
+#  endif
+    VALUES1(listof(count));
+  } else if (uint_p(STACK_0)) {
+#  if defined(HAVE_IF_INDEXTONAME)
+    char name[IF_NAMESIZE];
+    begin_system_call();
+    if (NULL == if_indextoname(I_to_uint(STACK_0),name)) OS_error();
+    end_system_call();
+    VALUES1(asciz_to_string(name,GLO(misc_encoding)));
+#  else
+    pushSTACK(TheSubr(subr_self)->name);
+    fehler(error,GETTEXT("~S: no if_indextoname() at configure time"));
+#  endif
+  } else if (stringp(STACK_0)) {
+#  if defined(HAVE_IF_INDEXTONAME)
+    unsigned int idx;
+    with_string_0(STACK_0,GLO(misc_encoding),namez, {
+        begin_system_call();
+        if (0 == (idx = if_nametoindex(namez))) OS_error();
+        end_system_call();
+      });
+    VALUES1(uint_to_I(idx));
+#  else
+    pushSTACK(TheSubr(subr_self)->name);
+    fehler(error,GETTEXT("~S: no if_nametoindex() at configure time"));
+#  endif
+  } else fehler_string_integer(STACK_0);
+  skipSTACK(1);
+}
+#endif  /* net/if.h */
+/* ================== ifaddrs.h interface ================== */
+#if defined(HAVE_NET_IF_H) && defined(HAVE_IFADDRS_H) && defined(HAVE_GETIFADDRS) && defined(HAVE_FREEIFADDRS)
+DEFCHECKER(check_iff,prefix=IFF,bitmasks=both,UP BROADCAST DEBUG LOOPBACK \
+           POINTOPOINT NOTRAILERS RUNNING NOARP PROMISC ALLMULTI        \
+           OACTIVE SIMPLEX LINK0 LINK1 LINK2 ALTPHYS POLLING PPROMISC MONITOR \
+           STATICARP NEEDSGIANT                                         \
+           MASTER SLAVE MULTICAST PORTSEL AUTOMEDIA DYNAMIC)
+DEFUN(RAWSOCK:IFADDRS,) {
+  struct ifaddrs *ifap;
+  int count;
+  begin_system_call(); if(-1==getifaddrs(&ifap)) OS_error(); end_system_call();
+  for (count=0; ifap; ifap=ifap->ifa_next, count++) {
+    pushSTACK(asciz_to_string(ifap->ifa_name,GLO(misc_encoding)));
+    pushSTACK(check_iff_to_list(ifap->ifa_flags));
+    pushSTACK(ifap->ifa_addr ? sockaddr_to_lisp1(ifap->ifa_addr) : NIL);
+    pushSTACK(ifap->ifa_netmask ? sockaddr_to_lisp1(ifap->ifa_netmask) : NIL);
+    if (ifap->ifa_flags & IFF_BROADCAST)
+      if (ifap->ifa_flags & IFF_POINTOPOINT) {
+        pushSTACK(TheSubr(subr_self)->name);
+        fehler(error,GETTEXT("~S: both IFF_BROADCAST and IFF_POINTOPOINT set"));
+      } else pushSTACK(ifap->ifa_broadaddr
+                       ? sockaddr_to_lisp1(ifap->ifa_broadaddr) : NIL);
+    else if (ifap->ifa_flags & IFF_POINTOPOINT)
+      pushSTACK(ifap->ifa_dstaddr ? sockaddr_to_lisp1(ifap->ifa_dstaddr) : NIL);
+    else pushSTACK(NIL);
+    pushSTACK(ifap->ifa_data ? allocate_fpointer(ifap->ifa_data) : NIL);
+    funcall(`RAWSOCK::MAKE-IFADDRS`,6);
+    pushSTACK(value1);
+  }
+  begin_system_call(); freeifaddrs(ifap); end_system_call();
+  VALUES1(listof(count));
+}
+#endif  /* ifaddrs.h */
 /* ================== sys/socket.h interface ================== */
 DEFCHECKER(check_socket_domain,prefix=AF,default=AF_UNSPEC,             \
            UNSPEC UNIX LOCAL INET IMPLINK PUP CHAOS AX25 DATAKIT CCITT  \
@@ -707,7 +809,7 @@ DEFCHECKER(check_gai_ecode,prefix=EAI,default=,AGAIN BADFLAGS FAIL FAMILY \
 #endif
 nonreturning_function(static, error_eai, (int ecode)) {
 #if defined(HAVE_GAI_STRERROR) || defined(WIN32_NATIVE)
-  const char* msg = gai_strerror(ecode);
+  const char* msg = gai_strerror_f(ecode);
 #else
   const char* msg = strerror(ecode);
 #endif
@@ -810,11 +912,7 @@ DEFUN(RAWSOCK:GETADDRINFO, &key NODE SERVICE PROTOCOL SOCKTYPE FAMILY \
     pushSTACK(check_socket_domain_reverse(tmp->ai_family));
     pushSTACK(check_socket_type_reverse(tmp->ai_socktype));
     pushSTACK(check_socket_protocol_reverse(tmp->ai_protocol));
-    pushSTACK(allocate_bit_vector(Atype_8Bit,tmp->ai_addrlen));
-    begin_system_call();
-    memcpy(TheSbvector(STACK_0)->data,tmp->ai_addr,tmp->ai_addrlen);
-    end_system_call();
-    funcall(`RAWSOCK::MAKE-SA`,1); pushSTACK(value1);
+    pushSTACK(sockaddr_to_lisp(tmp->ai_addr,tmp->ai_addrlen));
     pushSTACK(tmp->ai_canonname == NULL ? NIL
               : asciz_to_string(tmp->ai_canonname,GLO(misc_encoding)));
     funcall(`RAWSOCK::MAKE-ADDRINFO`,6); pushSTACK(value1);
