@@ -5,11 +5,24 @@
 (in-package "EXT")
 (mapcar #'fmakunbound '(short-site-name long-site-name))
 
-(defun short-site-name () (or (sys::getenv "ORGANIZATION") "edit config.lisp"))
-(defun long-site-name () (or (sys::getenv "ORGANIZATION") "edit config.lisp"))
+(let ((cache nil))
+(defun short-site-name ()
+  (if cache cache
+      (setq cache
+            (or (getenv "ORGANIZATION")
+                (with-open-stream (s (make-pipe-input-stream "uname -n"))
+                  (read-line s)))))))
+
+(let ((cache nil))
+(defun long-site-name ()
+  (if cache cache
+      (setq cache
+            (or (getenv "ORGANIZATION")
+                (with-open-stream (s (make-pipe-input-stream "uname -a"))
+                  (read-line s)))))))
 
 (defparameter *editor* "vi" "The name of the editor.")
-(defun editor-name () (or (sys::getenv "EDITOR") *editor*))
+(defun editor-name () (or (getenv "EDITOR") *editor*))
 
 (defun edit-file (file)
   "(edit-file file) edits a file."
@@ -35,10 +48,17 @@
 ;; which browser do you use? (see `*browsers*' in clhs.lisp)
 ;; (setq *browser* :mozilla-remote)
 
+(defvar *impnotes-root-default* "http://clisp.cons.org/impnotes/")
+(defun impnotes-root ()
+  "This returns the root URL for the CLISP implementation notes.
+You can set the environment variable `IMPNOTES' or redefine this function
+in ~/.clisprc.  On win32 you can also use the Registry."
+  (or (getenv "IMPNOTES") *impnotes-root-default*))
+
 ;; Common Lisp HyperSpec access
 (defvar *clhs-root-default*)
 (defun clhs-root ()
   "This returns the root URL for the Common Lisp HyperSpec.
 You can set the environment variable `CLHSROOT' or redefine this function
 in ~/.clisprc.  On win32 you can also use the Registry."
-  (or (sys::getenv "CLHSROOT") *clhs-root-default*))
+  (or (getenv "CLHSROOT") *clhs-root-default*))
