@@ -53,10 +53,11 @@
 
 ;; Classes
 
-(eval-when (compile load eval)
-  (defmethod clos:validate-superclass ((class class) (superclass (eql clos::<stream>)))
-    (or (call-next-method)
-        (eq (clos:class-name class) 'fundamental-stream)))
+(defmethod clos:validate-superclass ((class class)
+                                     (superclass (eql clos::<stream>)))
+  (or (call-next-method)
+      (eq (clos:class-name class) 'fundamental-stream)))
+(ext:compiler-let ((clos::*allow-mixing-metaclasses* t))
   (let ((clos::*allow-mixing-metaclasses* t))
     (clos:defclass fundamental-stream (stream clos:standard-object)
       (($open :type boolean :initform t) ; whether the stream is open
